/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __platform_Socket_h__
#define __platform_Socket_h__

namespace fe
{

typedef int SockHandle;

/**	BSD socket address wrapper. */
class SockAddr
{
	public:
		SockAddr(void)
		{
			memset(&m_sockaddr, 0, sizeof(m_sockaddr));
			m_sockaddr.sin_family = AF_INET;
			setAddress(INADDR_ANY);
		}

		/** Access pointer to underlying (struct sockaddr *) */
		struct sockaddr *	ptr(void)
		{
			return (struct sockaddr *)&m_sockaddr;
		}
		/** Return size of address. */
		int					size(void)
		{
			return sizeof(m_sockaddr);
		}
		/** Set the port.  @todo add a by-service-name port set. */
		void				setPort(FE_UWORD port)
		{
			m_sockaddr.sin_port = htons((short)port);
		}
		/** Set the address. */
		void				setAddress(const U32 address)
		{
			m_sockaddr.sin_addr.s_addr = htonl(address);
		}
		/** Set the address.  This may either be the hostname or the
			dot notation address. */
		void				setAddress(const String &host)
		{
			m_sockaddr.sin_addr.s_addr = getIp4Address(host);
			if(m_sockaddr.sin_addr.s_addr == 0)
			{
				feX("fe::SockAddr::setAddress",
					"unknown host: %s", host.c_str());
			}
		}
		/** Return the hostname for the address. */
		String				hostname(void);
		FE_UWORD				port(void)
		{
			return (long)ntohs(m_sockaddr.sin_port);
		}
		/** Return the address. */
		static	U32			getIp4Address(const String &host)
		{
			U32 address;

			if (host.empty())
			{
				return 0;
			}

			// If host is in dotted notation...
			address = inet_addr(host.c_str());

#if OSD_OSTYPE==OSD_WIN32
			if (INADDR_NONE != address)
#else /* OSD_OSTYPE==OSD_WIN32 */
			if ((U32) -1 != address)
#endif
			{
				return address;
			}

			// If host is *not* in dotted notation...
			register struct hostent *h = gethostbyname(host.c_str());
			if ((struct hostent *) 0 != h)
			{
				address = *((U32 *) (h->h_addr_list[0]));
				return address;
			}

			return 0;

		}

	protected:
		struct sockaddr_in	m_sockaddr;
};

/**	A socket wrapper.  The premise of this wrapper is to be simple and
	to the point.  Flexibility and extensibility are not explicitly considered.
	The intent is that all network communication will go through class and does
	not care about TCP/IP details.

	A concept of thread poison is supported by this class due to the blocking
	calls (accept, read, connect).  If Poison is associated with the Socket,
	then blocking calls will be aborted if the Poison becomes active before
	the calls can return normally.  If this happens then a Poisoned exception
	is thrown.

	A concept of network identifier is also supported by this class to provide
	a notion of site uniqueness in a networked application.  It is implemented
	in this lower level abstraction to assure simplicity in maintaining
	uniqueness.  A socket's unique id is 0 until it does a connect() in which
	case it will be given a unique id from the other side of the connection
	(presumably a 'server').
	*/
class Socket : public Counted, ObjectSafe<Socket>
{
	public:
			/** Constructor. */
						Socket(void);
			/** Construct a socket with the given poison. */
						Socket(Poison *pPoison);
virtual					~Socket(void);
			/** Wraps bsd socket(AF_INET, SOCK_STREAM, 0) call. */
virtual		void		openStream(void);
			/** Wraps bsd socket() call. */
virtual		SockHandle	socket(int domain, int type, int protocol);

			/** Wraps bsd bind() call. */
virtual		void		bind(SockAddr &sockaddr);
			/** Wraps bsd listen() call. */
virtual		void		listen(int backlog);
			/** Wraps bsd accept() call. */
virtual		sp<Socket>	accept(SockAddr &sockaddr);
			/** Wraps bsd connect() call. */
virtual		void		connect(SockAddr &sockaddr);

			/** Wraps bsd recv() call. */
virtual		IWORD		read(const void *pBuffer, IWORD count);
			/** Wraps bsd send() call. */
virtual		IWORD		write(const void *pBuffer, IWORD count);

			/** Set a Poison object. */
virtual		void		setPoison(Poison *pPoison);
			/** Wait for the socket to be readable (via select), checking at
				a regular interval for Poison to be active.  Mainly for
				internal use by read(), accept(), and connect() */
virtual		void		wait(void);

			/** Close the underlying socket. */
virtual		void		close(void);

			/** Return true if the socket is poen, otherwise return false. */
virtual		bool		isOpen(void);

			/** Return a network identifier unique among Sockets connected
				to the same accept()ing 'server' Socket.  Note that this has
				nothing to do with a socket descriptor. */
virtual		U32			id(void);

			/**	Startup socket networking.  Only required on Win32, but should
				always be done for portability. */
static		void		startup(void);
			/**	Shutdown socket networking. */
static		void		shutdown(void);

	private:
		SockHandle	m_sock_handle;
		bool		m_open;
		Poison		*m_pPoison;
		U32			m_id;
		U32			m_nextID;
};

} /* namespace */

#endif /* __platform_Socket_h__ */

